{% extends "base.html" %} {% block htmltitle %} {% if page.meta and
page.meta.title %}
<title>{{ page.meta.title }} | {{ config.site_name }}</title>
{% elif page.title and not page.is_homepage %}
<title>{{ page.title | striptags }} | {{ config.site_name }}</title>
{% else %}
<title>{{ config.site_name }}</title>
{% endif %} {% endblock %} {% block extrahead %}
<link
  rel="apple-touch-icon"
  sizes="180x180"
  href="https://docs.astral.sh/static/apple-touch-icon.png"
/>
<link
  rel="icon"
  type="image/png"
  sizes="32x32"
  href="https://docs.astral.sh/static/favicon-32x32.png"
/>
<link
  rel="icon"
  type="image/png"
  sizes="16x16"
  href="https://docs.astral.sh/static/favicon-16x16.png"
/>
<link rel="manifest" href="https://docs.astral.sh/static/site.webmanifest" />
<link
  rel="mask-icon"
  href="https://docs.astral.sh/static/safari-pinned-tab.svg"
  color="#2e183d"
/>
<meta name="msapplication-TileColor" content="#d7ff64" />
<meta name="theme-color" content="#ffffff" />
<meta name="robots" content="index,follow" />
<script type="application/ld+json">
  {
      {% if page and page.meta.git_revision_date_localized_raw_iso_datetime %}
      "datePublished": "{{ page.meta.git_revision_date_localized_raw_iso_datetime }}Z",
      "dateModified": "{{ page.meta.git_revision_date_localized_raw_iso_datetime  }}Z",
      {% endif %}
      "@context": "https://schema.org",
      "@type": "WebSite",
      "name": "Astral Docs",
      "url": "https://docs.astral.sh"
  }
</script>
{% endblock %}
